#include <sys/socket.h>
#include <netinet/in.h>
#include <sys/wait.h>
#include <signal.h>
#include <netdb.h>

#include <unistd.h>
#include <stdlib.h>
#include <errno.h>

/* Copyright (C) 1992,2000 Bisqwit (http://iki.fi/bisqwit/) *
 * No warranty. License: see file COPYING.                  */

#include "tcpip.h"

static int ConnectOutTimed;

static void SubConnectOut(int signum) { signum = ConnectOutTimed++; }

int ConnectOut(const char *Server, int Port, unsigned time)
{
    struct hostent *HostName;
    struct sockaddr_in SocketName;
    int plug, p;
    
    if((plug = socket(AF_INET,SOCK_STREAM,0)) < 0
    || !(HostName = gethostbyname(Server)))return -1;
    
    SocketName.sin_family = AF_INET;
    SocketName.sin_port = htons(Port);
    memcpy(&SocketName.sin_addr, HostName->h_addr, HostName->h_length);
	
    signal(SIGABRT, SubConnectOut);
	
    if((p = fork()) < 0)return -1;
	
    ConnectOutTimed = 0;
    /* This timeout implementation doesn't eat tons of
     * CPU time looping and doesn't clobber queued alarms.
     * It, however, resets the SIGABRT signal handler. */
    if(!p)
    {
		usleep(time);
        kill(getppid(),SIGABRT);
        _exit(EXIT_SUCCESS);
    } 
    for(;;)
    {
        if((connect(plug,(struct sockaddr *)&SocketName,sizeof(SocketName))) < 0)
        {
        	/* If ConnectOutTimed>0, then errno can contain whatever. */
        	if(errno == EINTR && !ConnectOutTimed)continue;
            if(ConnectOutTimed)errno = ETIMEDOUT;
            close(plug);
            plug = -1;
        }        
        else
        {
        	signal(SIGPIPE, SIG_IGN);
        }
        kill(p, SIGKILL);
        signal(SIGABRT, SIG_DFL);
        waitpid(p, NULL, 0);
        return plug;
    }
}
